﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis {
  [Item("Transformation", "Represents the base class for transformations.")]
  [StorableClass]
  public abstract class Transformation<T> : ParameterizedNamedItem, ITransformation<T> {
    #region Constructor, Cloning & Persistence
    protected Transformation()
      : base() {
    }

    protected Transformation(Transformation<T> original, Cloner cloner)
      : base(original, cloner) {
    }

    [StorableConstructor]
    protected Transformation(bool deserializing)
      : base(deserializing) {
    }
    #endregion

    public virtual bool Check(IEnumerable<T> data, out string errorMessage) {
      errorMessage = string.Empty;
      return true;
    }

    public virtual void Configure(IEnumerable<T> data) {
    }

    public abstract IEnumerable<T> Apply(IEnumerable<T> data);

    public abstract IEnumerable<T> InverseApply(IEnumerable<T> data);
  }
}